package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.internal.properties.AdventureProperties;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.util.Nag;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.VisibleForTesting;

final class TextComponentImpl extends AbstractComponent implements TextComponent {
   private static final boolean WARN_WHEN_LEGACY_FORMATTING_DETECTED;
   @VisibleForTesting
   static final char SECTION_CHAR = '§';
   static final TextComponent EMPTY;
   static final TextComponent NEWLINE;
   static final TextComponent SPACE;
   private final String content;

   static TextComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final @NotNull String content) {
      List<Component> filteredChildren = ComponentLike.asComponents(children, IS_NOT_EMPTY);
      return (TextComponent)(filteredChildren.isEmpty() && style.isEmpty() && content.isEmpty() ? Component.empty() : new TextComponentImpl(filteredChildren, (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(content, "content")));
   }

   private static @NotNull TextComponent createDirect(final @NotNull String content) {
      return new TextComponentImpl(Collections.emptyList(), Style.empty(), content);
   }

   TextComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String content) {
      super(children, style);
      this.content = content;
      if (WARN_WHEN_LEGACY_FORMATTING_DETECTED) {
         LegacyFormattingDetected nag = this.warnWhenLegacyFormattingDetected();
         if (nag != null) {
            Nag.print(nag);
         }
      }

   }

   @VisibleForTesting
   final @Nullable LegacyFormattingDetected warnWhenLegacyFormattingDetected() {
      return this.content.indexOf(167) != -1 ? new LegacyFormattingDetected(this) : null;
   }

   public @NotNull String content() {
      return this.content;
   }

   public @NotNull TextComponent content(final @NotNull String content) {
      return (TextComponent)(Objects.equals(this.content, content) ? this : create(this.children, this.style, content));
   }

   public @NotNull TextComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.content);
   }

   public @NotNull TextComponent style(final @NotNull Style style) {
      return create(this.children, style, this.content);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof TextComponentImpl)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         TextComponentImpl that = (TextComponentImpl)other;
         return Objects.equals(this.content, that.content);
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.content.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   @NotNull
   public TextComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static {
      WARN_WHEN_LEGACY_FORMATTING_DETECTED = Boolean.TRUE.equals(AdventureProperties.TEXT_WARN_WHEN_LEGACY_FORMATTING_DETECTED.value());
      EMPTY = createDirect("");
      NEWLINE = createDirect("\n");
      SPACE = createDirect(" ");
   }

   static final class BuilderImpl extends AbstractComponentBuilder<TextComponent, TextComponent.Builder> implements TextComponent.Builder {
      private String content = "";

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull TextComponent component) {
         super(component);
         this.content = component.content();
      }

      @NotNull
      public TextComponent.@NotNull Builder content(final @NotNull String content) {
         this.content = (String)Objects.requireNonNull(content, "content");
         return this;
      }

      public @NotNull String content() {
         return this.content;
      }

      public @NotNull TextComponent build() {
         return this.isEmpty() ? Component.empty() : TextComponentImpl.create(this.children, this.buildStyle(), this.content);
      }

      private boolean isEmpty() {
         return this.content.isEmpty() && this.children.isEmpty() && !this.hasStyle();
      }
   }
}
